"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.mergeConfigs = mergeConfigs;
var _FaceCameraLensDirection = require("../config/enums/FaceCameraLensDirection");
var _FaceImageFormat = require("../config/enums/FaceImageFormat");
var _FaceMaskFormat = require("../config/enums/FaceMaskFormat");
var _FaceResolutionPreset = require("../config/enums/FaceResolutionPreset");
const defaultConfig = {
  licenseKey: '',
  resolutionPreset: _FaceResolutionPreset.FaceResolutionPreset.VERYHIGH,
  lensDirection: _FaceCameraLensDirection.FaceCameraLensDirection.FRONT,
  imageFormat: _FaceImageFormat.FaceImageFormat.JPEG,
  flashEnabled: false,
  fontFamily: 'facesdk_opensans_bold',
  liveness: {
    enabled: false,
    debug: false,
    timeToCapture: 3000,
    maxFaceDetectionTime: 60000,
    minFaceProp: 0.1,
    maxFaceProp: 0.4,
    minFaceWidth: 150,
    minFaceHeight: 150,
    ied: 90,
    bboxPad: 20,
    faceDetectionThresh: 0.5,
    rollThresh: 4.0,
    pitchThresh: 4.0,
    yawThresh: 4.0,
    closedEyesThresh: 0.7,
    smilingThresh: 0.7,
    tooDarkThresh: 50,
    tooLightThresh: 170,
    faceCentralizationThresh: 0.05
  },
  continuousCapture: {
    enabled: false,
    timeToCapture: 1000,
    maxNumberFrames: 40
  },
  faceDetection: {
    enabled: true,
    autoCapture: true,
    multipleFacesEnabled: false,
    timeToCapture: 3000,
    maxFaceDetectionTime: 40000,
    scoreThreshold: 0.7
  },
  mask: {
    enabled: true,
    type: _FaceMaskFormat.FaceMaskFormat.FACE,
    backgroundColor: '#CC000000',
    frameColor: '#FFFFFF',
    frameEnabledColor: '#16AC81',
    frameErrorColor: '#E25353'
  },
  titleText: {
    enabled: true,
    content: 'Capturing Face',
    textColor: '#FFFFFF',
    textSize: 20
  },
  loadingText: {
    enabled: true,
    content: 'Processing...',
    textColor: '#FFFFFF',
    textSize: 14
  },
  helpText: {
    enabled: true,
    content: 'Fit your face into the shape below',
    textColor: '#FFFFFF',
    textSize: 14
  },
  feedbackText: {
    enabled: true,
    messages: {
      noDetection: 'No faces detected',
      multipleFaces: 'Multiple faces detected',
      faceCentered: 'Face centered. Do not move',
      tooClose: 'Turn your face away',
      tooFar: 'Bring your face closer',
      tooLeft: 'Move your face to the right',
      tooRight: 'Move your face to the left',
      tooUp: 'Move your face down',
      tooDown: 'Move your face up',
      invalidIED: 'Invalid inter-eye distance',
      faceAngleMisaligned: 'Misaligned face angle',
      closedEyes: 'Open your eyes',
      smiling: 'Do not smile',
      tooDark: 'Too dark',
      tooLight: 'Too light'
    },
    textColor: '#FFFFFF',
    textSize: 14
  },
  backButton: {
    enabled: true,
    backgroundColor: '#00000000',
    buttonPadding: 0,
    buttonSize: {
      width: 56,
      height: 56
    },
    iconOptions: {
      enabled: true,
      iconFile: 'facesdk_ic_close',
      iconColor: '#FFFFFF',
      iconSize: {
        width: 32,
        height: 32
      }
    },
    labelOptions: {
      enabled: false,
      content: 'Back',
      textColor: '#FFFFFF',
      textSize: 14
    }
  },
  flashButton: {
    enabled: false,
    backgroundColor: '#FFFFFF',
    buttonPadding: 0,
    buttonSize: {
      width: 56,
      height: 56
    },
    flashOnIconOptions: {
      enabled: true,
      iconFile: 'facesdk_ic_flash_on',
      iconColor: '#FFCC01',
      iconSize: {
        width: 32,
        height: 32
      }
    },
    flashOnLabelOptions: {
      enabled: false,
      content: 'Flash On',
      textColor: '#323232',
      textSize: 14
    },
    flashOffIconOptions: {
      enabled: true,
      iconFile: 'facesdk_ic_flash_off',
      iconColor: '#323232',
      iconSize: {
        width: 32,
        height: 32
      }
    },
    flashOffLabelOptions: {
      enabled: false,
      content: 'Flash Off',
      textColor: '#323232',
      textSize: 14
    }
  },
  switchCameraButton: {
    enabled: true,
    backgroundColor: '#FFFFFF',
    buttonPadding: 0,
    buttonSize: {
      width: 56,
      height: 56
    },
    iconOptions: {
      enabled: true,
      iconFile: 'facesdk_ic_switch_camera',
      iconColor: '#323232',
      iconSize: {
        width: 32,
        height: 32
      }
    },
    labelOptions: {
      enabled: false,
      content: 'Switch Camera',
      textColor: '#323232',
      textSize: 14
    }
  },
  captureButton: {
    enabled: true,
    backgroundColor: '#FFFFFF',
    buttonPadding: 0,
    buttonSize: {
      width: 56,
      height: 56
    },
    iconOptions: {
      enabled: true,
      iconFile: 'facesdk_ic_capture',
      iconColor: '#323232',
      iconSize: {
        width: 32,
        height: 32
      }
    },
    labelOptions: {
      enabled: false,
      content: 'Capture',
      textColor: '#323232',
      textSize: 14
    }
  }
};
function mergeConfigs(config) {
  var _defaultConfig$feedba, _config$feedbackText, _defaultConfig$backBu, _config$backButton, _defaultConfig$backBu2, _config$backButton2, _defaultConfig$backBu3, _config$backButton3, _defaultConfig$backBu4, _config$backButton4, _defaultConfig$flashB, _config$flashButton, _defaultConfig$flashB2, _config$flashButton2, _defaultConfig$flashB3, _config$flashButton3, _defaultConfig$flashB4, _config$flashButton4, _defaultConfig$flashB5, _config$flashButton5, _defaultConfig$flashB6, _config$flashButton6, _defaultConfig$flashB7, _config$flashButton7, _defaultConfig$switch, _config$switchCameraB, _defaultConfig$switch2, _config$switchCameraB2, _defaultConfig$switch3, _config$switchCameraB3, _defaultConfig$switch4, _config$switchCameraB4, _defaultConfig$captur, _config$captureButton, _defaultConfig$captur2, _config$captureButton2, _defaultConfig$captur3, _config$captureButton3, _defaultConfig$captur4, _config$captureButton4;
  return {
    ...defaultConfig,
    ...config,
    liveness: {
      ...defaultConfig.liveness,
      ...config.liveness
    },
    continuousCapture: {
      ...defaultConfig.continuousCapture,
      ...config.continuousCapture
    },
    faceDetection: {
      ...defaultConfig.faceDetection,
      ...config.faceDetection
    },
    mask: {
      ...defaultConfig.mask,
      ...config.mask
    },
    titleText: {
      ...defaultConfig.titleText,
      ...config.titleText
    },
    loadingText: {
      ...defaultConfig.loadingText,
      ...config.loadingText
    },
    helpText: {
      ...defaultConfig.helpText,
      ...config.helpText
    },
    feedbackText: {
      ...defaultConfig.feedbackText,
      ...config.feedbackText,
      messages: {
        ...((_defaultConfig$feedba = defaultConfig.feedbackText) === null || _defaultConfig$feedba === void 0 ? void 0 : _defaultConfig$feedba.messages),
        ...((_config$feedbackText = config.feedbackText) === null || _config$feedbackText === void 0 ? void 0 : _config$feedbackText.messages)
      }
    },
    backButton: {
      ...defaultConfig.backButton,
      ...config.backButton,
      buttonSize: {
        ...((_defaultConfig$backBu = defaultConfig.backButton) === null || _defaultConfig$backBu === void 0 ? void 0 : _defaultConfig$backBu.buttonSize),
        ...((_config$backButton = config.backButton) === null || _config$backButton === void 0 ? void 0 : _config$backButton.buttonSize)
      },
      iconOptions: {
        ...((_defaultConfig$backBu2 = defaultConfig.backButton) === null || _defaultConfig$backBu2 === void 0 ? void 0 : _defaultConfig$backBu2.iconOptions),
        ...((_config$backButton2 = config.backButton) === null || _config$backButton2 === void 0 ? void 0 : _config$backButton2.iconOptions),
        iconSize: {
          ...((_defaultConfig$backBu3 = defaultConfig.backButton) === null || _defaultConfig$backBu3 === void 0 || (_defaultConfig$backBu3 = _defaultConfig$backBu3.iconOptions) === null || _defaultConfig$backBu3 === void 0 ? void 0 : _defaultConfig$backBu3.iconSize),
          ...((_config$backButton3 = config.backButton) === null || _config$backButton3 === void 0 || (_config$backButton3 = _config$backButton3.iconOptions) === null || _config$backButton3 === void 0 ? void 0 : _config$backButton3.iconSize)
        }
      },
      labelOptions: {
        ...((_defaultConfig$backBu4 = defaultConfig.backButton) === null || _defaultConfig$backBu4 === void 0 ? void 0 : _defaultConfig$backBu4.labelOptions),
        ...((_config$backButton4 = config.backButton) === null || _config$backButton4 === void 0 ? void 0 : _config$backButton4.labelOptions)
      }
    },
    flashButton: {
      ...defaultConfig.flashButton,
      ...config.flashButton,
      buttonSize: {
        ...((_defaultConfig$flashB = defaultConfig.flashButton) === null || _defaultConfig$flashB === void 0 ? void 0 : _defaultConfig$flashB.buttonSize),
        ...((_config$flashButton = config.flashButton) === null || _config$flashButton === void 0 ? void 0 : _config$flashButton.buttonSize)
      },
      flashOnIconOptions: {
        ...((_defaultConfig$flashB2 = defaultConfig.flashButton) === null || _defaultConfig$flashB2 === void 0 ? void 0 : _defaultConfig$flashB2.flashOnIconOptions),
        ...((_config$flashButton2 = config.flashButton) === null || _config$flashButton2 === void 0 ? void 0 : _config$flashButton2.flashOnIconOptions),
        iconSize: {
          ...((_defaultConfig$flashB3 = defaultConfig.flashButton) === null || _defaultConfig$flashB3 === void 0 || (_defaultConfig$flashB3 = _defaultConfig$flashB3.flashOnIconOptions) === null || _defaultConfig$flashB3 === void 0 ? void 0 : _defaultConfig$flashB3.iconSize),
          ...((_config$flashButton3 = config.flashButton) === null || _config$flashButton3 === void 0 || (_config$flashButton3 = _config$flashButton3.flashOnIconOptions) === null || _config$flashButton3 === void 0 ? void 0 : _config$flashButton3.iconSize)
        }
      },
      flashOnLabelOptions: {
        ...((_defaultConfig$flashB4 = defaultConfig.flashButton) === null || _defaultConfig$flashB4 === void 0 ? void 0 : _defaultConfig$flashB4.flashOnLabelOptions),
        ...((_config$flashButton4 = config.flashButton) === null || _config$flashButton4 === void 0 ? void 0 : _config$flashButton4.flashOnLabelOptions)
      },
      flashOffIconOptions: {
        ...((_defaultConfig$flashB5 = defaultConfig.flashButton) === null || _defaultConfig$flashB5 === void 0 ? void 0 : _defaultConfig$flashB5.flashOffIconOptions),
        ...((_config$flashButton5 = config.flashButton) === null || _config$flashButton5 === void 0 ? void 0 : _config$flashButton5.flashOffIconOptions),
        iconSize: {
          ...((_defaultConfig$flashB6 = defaultConfig.flashButton) === null || _defaultConfig$flashB6 === void 0 || (_defaultConfig$flashB6 = _defaultConfig$flashB6.flashOffIconOptions) === null || _defaultConfig$flashB6 === void 0 ? void 0 : _defaultConfig$flashB6.iconSize),
          ...((_config$flashButton6 = config.flashButton) === null || _config$flashButton6 === void 0 || (_config$flashButton6 = _config$flashButton6.flashOffIconOptions) === null || _config$flashButton6 === void 0 ? void 0 : _config$flashButton6.iconSize)
        }
      },
      flashOffLabelOptions: {
        ...((_defaultConfig$flashB7 = defaultConfig.flashButton) === null || _defaultConfig$flashB7 === void 0 ? void 0 : _defaultConfig$flashB7.flashOffLabelOptions),
        ...((_config$flashButton7 = config.flashButton) === null || _config$flashButton7 === void 0 ? void 0 : _config$flashButton7.flashOffLabelOptions)
      }
    },
    switchCameraButton: {
      ...defaultConfig.switchCameraButton,
      ...config.switchCameraButton,
      buttonSize: {
        ...((_defaultConfig$switch = defaultConfig.switchCameraButton) === null || _defaultConfig$switch === void 0 ? void 0 : _defaultConfig$switch.buttonSize),
        ...((_config$switchCameraB = config.switchCameraButton) === null || _config$switchCameraB === void 0 ? void 0 : _config$switchCameraB.buttonSize)
      },
      iconOptions: {
        ...((_defaultConfig$switch2 = defaultConfig.switchCameraButton) === null || _defaultConfig$switch2 === void 0 ? void 0 : _defaultConfig$switch2.iconOptions),
        ...((_config$switchCameraB2 = config.switchCameraButton) === null || _config$switchCameraB2 === void 0 ? void 0 : _config$switchCameraB2.iconOptions),
        iconSize: {
          ...((_defaultConfig$switch3 = defaultConfig.switchCameraButton) === null || _defaultConfig$switch3 === void 0 || (_defaultConfig$switch3 = _defaultConfig$switch3.iconOptions) === null || _defaultConfig$switch3 === void 0 ? void 0 : _defaultConfig$switch3.iconSize),
          ...((_config$switchCameraB3 = config.switchCameraButton) === null || _config$switchCameraB3 === void 0 || (_config$switchCameraB3 = _config$switchCameraB3.iconOptions) === null || _config$switchCameraB3 === void 0 ? void 0 : _config$switchCameraB3.iconSize)
        }
      },
      labelOptions: {
        ...((_defaultConfig$switch4 = defaultConfig.switchCameraButton) === null || _defaultConfig$switch4 === void 0 ? void 0 : _defaultConfig$switch4.labelOptions),
        ...((_config$switchCameraB4 = config.switchCameraButton) === null || _config$switchCameraB4 === void 0 ? void 0 : _config$switchCameraB4.labelOptions)
      }
    },
    captureButton: {
      ...defaultConfig.captureButton,
      ...config.captureButton,
      buttonSize: {
        ...((_defaultConfig$captur = defaultConfig.captureButton) === null || _defaultConfig$captur === void 0 ? void 0 : _defaultConfig$captur.buttonSize),
        ...((_config$captureButton = config.captureButton) === null || _config$captureButton === void 0 ? void 0 : _config$captureButton.buttonSize)
      },
      iconOptions: {
        ...((_defaultConfig$captur2 = defaultConfig.captureButton) === null || _defaultConfig$captur2 === void 0 ? void 0 : _defaultConfig$captur2.iconOptions),
        ...((_config$captureButton2 = config.captureButton) === null || _config$captureButton2 === void 0 ? void 0 : _config$captureButton2.iconOptions),
        iconSize: {
          ...((_defaultConfig$captur3 = defaultConfig.captureButton) === null || _defaultConfig$captur3 === void 0 || (_defaultConfig$captur3 = _defaultConfig$captur3.iconOptions) === null || _defaultConfig$captur3 === void 0 ? void 0 : _defaultConfig$captur3.iconSize),
          ...((_config$captureButton3 = config.captureButton) === null || _config$captureButton3 === void 0 || (_config$captureButton3 = _config$captureButton3.iconOptions) === null || _config$captureButton3 === void 0 ? void 0 : _config$captureButton3.iconSize)
        }
      },
      labelOptions: {
        ...((_defaultConfig$captur4 = defaultConfig.captureButton) === null || _defaultConfig$captur4 === void 0 ? void 0 : _defaultConfig$captur4.labelOptions),
        ...((_config$captureButton4 = config.captureButton) === null || _config$captureButton4 === void 0 ? void 0 : _config$captureButton4.labelOptions)
      }
    }
  };
}
//# sourceMappingURL=Utils.js.map